;***************************************************
;                  Ansteuerung fr        
;           LCD 4 Lines x 16 Characters 
;     HLM8070 (display controller M50530-XXXFP)
;     Beispiel gibt
;     	 	A
;			B
;			C
;			D
;	  auf dem Display aus (im 8-bit Modus)		
;***************************************************
;		  mega8 PIN	 LCD-Pin	Funktion
;                         
;      				  1		GND           
;      				  2		VDD +5V       
;      				  3		Kontrast      
;      		PC0	23	  4		I/OC1         
;      		PC1	24	  5		R/W           
;      		PC2	25	  6		ex (enable)            
;      		PD0	 2	  7		DB0           
;      		PD1	 3	  8		DB1           
;      		PD2	 4    9		DB2           
;      		PD3	 5	 10		DB3          
;      		PD4	 6	 11		DB4          
;      		PD5	11	 12		DB5          
;      		PD6	12	 13		DB6          
;      		PD7	13   14		DB7          
;      		PC3	26	 15		I/OC2        


		.INCLUDE "m8def.inc"

		.EQU bLCD_IOC1   = 0		;	LCD Steuerbit1
		.EQU bLCD_RW     = 1		;	LCD Lesen/Schreiben 
		.EQU bLCD_Enable = 2		;	LCD Enable
		.EQU bLCD_IOC2   = 3		;	LCD Steuerbit2
		
		.DEF akku = r16
		
		.CSEG					; Programm - Flash

rjmp INIT				; Einsprung nach Reset
			
		.ORG $2A


INIT:	
		ldi	akku, LOW(RAMEND)	; Stapel anlegen
		out	SPL, akku
		ldi akku, HIGH(RAMEND)
		out SPH, akku
		
		rcall LCD_INIT
		rcall W20mS

;"ABCD" schreiben
		
		rcall W192uS			; Zeitschleife
		rcall SetLine1			; Display auf erste Zeile setzen		
		rcall W192uS			; 
		ldi r17, 'A'			; 	 
		rcall Wchar				; A schreiben

		rcall W192uS
		rcall SetLine2
		rcall W192uS
		ldi r17, 'B'			 
		rcall Wchar

		rcall W192uS
		rcall SetLine3
		rcall W192uS
		ldi r17, 'C'			 
		rcall Wchar

		rcall W192uS
		rcall SetLine4
		rcall W192uS
		ldi r17, 'D'			 
		rcall Wchar
		
ende:	rjmp ende

LCD_INIT:
		ldi akku, 0x00
		out PORTC, akku 		; LCD Steuerbits 0
		sbi DDRC, bLCD_IOC1		; Als Ausgang		
		sbi DDRC, bLCD_IOC2		; Als Ausgang
		sbi DDRC, bLCD_RW		; Als Ausgang
		sbi DDRC, bLCD_Enable	; Als Ausgang
		ldi akku, 0xFF
		out DDRD, akku			; PortD(Daten) als Ausgang
		
                                ; SET DISPLAY MODE
								;--------------------------------------------------------
		rcall W192uS		    ;|IOC1|IOC2| RW |DB7 |DB6 |DB5 |DB4 |DB3 |DB2 |DB1 |DB0 | 
		ldi r17, 0b00110000		;|------------------------------------------------------|					
		rcall Wcom			    ;|    |    |    |    |    |    |DSP |CSR |UND |CSR |CHAR|				
								;| 0  | 0  | 0  | 0  | 0  | 1  |ON  |ON  |ON  |BLK |BLK |
								;|    |    |    |    |    |    |OFF |OFF |OFF |    |    |
								;--------------------------------------------------------
													
		                        ; SET FUNKTION MODE
								;--------------------------------------------------------
		rcall W192uS            ;|IOC1|IOC2| RW |DB7 |DB6 |DB5 |DB4 |DB3 |DB2 |DB1 |DB0 |
		ldi r17, 0b11111000		;|------------------------------------------------------|
		rcall Wcom              ;|    |    |    |    |    |I/0 |FONT|  DUTY   |   RAM   |
		                        ;| 0  | 0  | 0  | 1  | 1  |8/4 |8/12|DT1 |DT0 |RA1 |RA0 | 1/0
		                        ;|    |    |    |    |    |    |DOTS|    |    |    |    |
		                        ;--------------------------------------------------------
		                        
		                        ; SET ENTRY MODE
		                        ;--------------------------------------------------------    
		rcall W192uS            ;|IOC1|IOC2| RW |DB7 |DB6 |DB5 |DB4 |DB3 |DB2 |DB1 |DB0 |    
		ldi r17, 0b01011000		;|------------------------------------------------------|    
		rcall Wcom              ;|    |    |    |    |    |CSR |CSR COND |DSP |DSP COND |    
                                ;| 0  | 0  | 0  | 0  | 1  |DEC |WR  |READ|DEC |WR  |READ| 1
		                        ;|    |    |    |    |    |INC |    |    |INC |    |    | 0  
		                        ;--------------------------------------------------------    
		                        	                        
		                        ; SET UNDERLINE MODE                                       
		                        ;--------------------------------------------------------  
		rcall W192uS            ;|IOC1|IOC2| RW |DB7 |DB6 |DB5 |DB4 |DB3 |DB2 |DB1 |DB0 |  
		ldi r17, 0b00001100		;|------------------------------------------------------|  
		rcall Wcom              ;|    |    |    |    |    |    |    |    |    |CSR |UND |  
                                ;| 0  | 0  | 0  | 0  | 0  | 0  | 0  | 1  | 1  |ON  |S/  | 1
		                        ;|    |    |    |    |    |    |    |    |    |OFF |R   | 0
		                        ;--------------------------------------------------------
		                        
		                        ; SET BLINKING FREQUENCY
		                        ;--------------------------------------------------------    
		rcall W192uS            ;|IOC1|IOC2| RW |DB7 |DB6 |DB5 |DB4 |DB3 |DB2 |DB1 |DB0 |  
		ldi r17, 0b00000110		;|------------------------------------------------------|  
		rcall Wcom              ;|    |    |    |    |    |    |    |    |DSP |BLK FREQ |  
		                        ;| 0  | 0  | 0  | 0  | 0  | 0  | 0  | 0  | 1  | B1 | B0 | 
		                        ;|    |    |    |    |    |    |    |    |    |    |    | 
		                        ;--------------------------------------------------------
		                          		                        
		                        ; RAM DATA CLEAR CURSER HOME                              
		                        ;--------------------------------------------------------  
		rcall W192uS            ;|IOC1|IOC2| RW |DB7 |DB6 |DB5 |DB4 |DB3 |DB2 |DB1 |DB0 |  
		ldi r17, 0b00000001		;|------------------------------------------------------|  
		rcall Wcom              ;|    |    |    |    |    |    |    |    |    |    |    |  
		ret                     ;| 0  | 0  | 0  | 0  | 0  | 0  | 0  | 0  | 0  | 0  | 1  | 
		                        ;|    |    |    |    |    |    |    |    |    |    |    | 
		                        ;--------------------------------------------------------  
		
		
Wcom:                           ; WRITE CURSOR ADDRESS                                   
		cbi PORTC, bLCD_IOC1	;--------------------------------------------------------
		cbi PORTC, bLCD_IOC2	;|IOC1|IOC2| RW |DB7 |DB6 |DB5 |DB4 |DB3 |DB2 |DB1 |DB0 |
		cbi PORTC, bLCD_RW		;|------------------------------------------------------|
		out PORTD, r17          ;|    |    |    |            BEFEHL                     |
		nop                     ;| 0  | 0  | 0  |    |    |    |    |    |    |    |    |
		sbi PORTC, bLCD_Enable	;|    |    |    |    |    |    |    |    |    |    |    |
		nop                     ;--------------------------------------------------------
		nop
		nop
		cbi PORTC, bLCD_Enable
		nop
		nop
		nop
		ldi akku, 0x00
		out PORTD, akku
		ret

Wchar:                          ; WRITE RAM DATA                                   
		cbi PORTC, bLCD_IOC1	;--------------------------------------------------------
		sbi PORTC, bLCD_IOC2	;|IOC1|IOC2| RW |DB7 |DB6 |DB5 |DB4 |DB3 |DB2 |DB1 |DB0 |
		cbi PORTC, bLCD_RW		;|------------------------------------------------------|
		out PORTD, r17          ;|    |    |    |                DATEN                  |
		nop                     ;| 0  | 1  | 0  |    |    |    |    |    |    |    |    |
		sbi PORTC, bLCD_Enable	;|    |    |    |    |    |    |    |    |    |    |    |
		nop                     ;--------------------------------------------------------
		nop
		nop
		cbi PORTC, bLCD_Enable
		nop
		nop
		nop
		cbi PORTC, bLCD_IOC2	;
		ldi akku, 0x00
		out PORTD, akku		
		ret

SetLine1:
		ldi r17, 0x00			; Cursoradresse 1.Zeile
		rcall SetLine
		ret

SetLine2:
		ldi r17, 0x40			; Cursoradresse 2.Zeile
		rcall SetLine
		ret

SetLine3:
		ldi r17, 0x80			; Cursoradresse 3.Zeile
		rcall SetLine
		ret

SetLine4:
		ldi r17, 0xC0			; Cursoradresse 4.Zeile
		rcall SetLine
		ret

SetLine:                        ; WRITE CURSOR ADDRESS 
		sbi PORTC, bLCD_IOC1	;--------------------------------------------------------
		sbi PORTC, bLCD_IOC2	;|IOC1|IOC2| RW |DB7 |DB6 |DB5 |DB4 |DB3 |DB2 |DB1 |DB0 |
		cbi PORTC, bLCD_RW		;|------------------------------------------------------|
		out PORTD, r17          ;|    |    |    |              ADRESSE                  |
		nop                     ;| 1  | 1  | 0  |    |    |    |    |    |    |    |    |
		sbi PORTC, bLCD_Enable	;|    |    |    |    |    |    |    |    |    |    |    |
		nop                     ;--------------------------------------------------------
		nop
		nop
		cbi PORTC, bLCD_Enable
		nop
		nop
		nop
		cbi PORTC, bLCD_IOC1	;
		cbi PORTC, bLCD_IOC2	;
		ldi akku, 0x00
		out PORTD, akku		
		ret

W192uS:	ldi r18, 0xFF
A1:		dec r18
		brne A1			; wenn r18 nicht 0
		ret
		
W20ms:	ldi r19, 0x67
A2:		rcall W192uS
		dec r19
		brne A2			; wenn r19 nicht 0	
		ret
